@@ -5,7 +5,7 @@ from lbmpy.advanced_streaming.utility import AccessPdfValues, Timestep
 from pystencils.simp.assignment_collection import AssignmentCollection
 from pystencils import Assignment, Field
 from lbmpy.boundaries.boundaryhandling import LbmWeightInfo
-from pystencils.data_types import create_type
+from pystencils.typing import create_type
 from pystencils.sympyextensions import get_symmetric_part
 from lbmpy.simplificationfactory import create_simplification_strategy
 from lbmpy.advanced_streaming.indexing import NeighbourOffsetArrays, MirroredStencilDirections
@@ -120,9 +120,10 @@ class FreeSlip(LbBoundary):
 
     Args:
         stencil: LBM stencil which is used for the simulation
-        normal_direction: optional normal direction. If the Free slip boundary is applied to a certain side in the
-                          domain it is not necessary to calculate the normal direction since it can be stated for all
-                          boundary cells. This reduces the memory space for the index array significantly.
+        normal_direction: optional normal direction pointing from wall to fluid.
+                          If the Free slip boundary is applied to a certain side in the domain it is not necessary
+                          to calculate the normal direction since it can be stated for all boundary cells.
+                          This reduces the memory space for the index array significantly.
         name: optional name of the boundary.
     """
 
@@ -182,7 +183,12 @@ class FreeSlip(LbBoundary):
                     normal_direction[i] = direction[i]
                     ref_direction = MirroredStencilDirections.mirror_stencil(ref_direction, i)
 
-            ref_direction = inverse_direction(ref_direction)
+            # convex corner special case:
+            if all(n == 0 for n in normal_direction):
+                normal_direction = direction
+            else:
+                ref_direction = inverse_direction(ref_direction)
+
             for i, cell_name in zip(range(dim), self.additional_data):
                 cell[cell_name[0]] = -normal_direction[i]
             cell['ref_dir'] = self.stencil.index(ref_direction)
@@ -208,13 +214,14 @@ class FreeSlip(LbBoundary):
 
     def get_additional_code_nodes(self, lb_method):
         if self.normal_direction:
-            return [MirroredStencilDirections(self.stencil, self.mirror_axis)]
+            return [MirroredStencilDirections(self.stencil, self.mirror_axis), NeighbourOffsetArrays(lb_method.stencil)]
         else:
-            return []
+            return [NeighbourOffsetArrays(lb_method.stencil)]
 
     def __call__(self, f_out, f_in, dir_symbol, inv_dir, lb_method, index_field):
+        neighbor_offset = NeighbourOffsetArrays.neighbour_offset(dir_symbol, lb_method.stencil)
         if self.normal_direction:
-            normal_direction = self.normal_direction
+            tangential_offset = tuple(offset + normal for offset, normal in zip(neighbor_offset, self.normal_direction))
             mirrored_stencil_symbol = MirroredStencilDirections._mirrored_symbol(self.mirror_axis)
             mirrored_direction = inv_dir[sp.IndexedBase(mirrored_stencil_symbol, shape=(1,))[dir_symbol]]
         else:
@@ -222,10 +229,11 @@ class FreeSlip(LbBoundary):
             for i, cell_name in zip(range(self.dim), self.additional_data):
                 normal_direction.append(index_field[0](cell_name[0]))
             normal_direction = tuple(normal_direction)
+            tangential_offset = tuple(offset + normal for offset, normal in zip(neighbor_offset, normal_direction))
 
             mirrored_direction = index_field[0]('ref_dir')
 
-        return Assignment(f_in(inv_dir[dir_symbol]), f_in[normal_direction](mirrored_direction))
+        return Assignment(f_in.center(inv_dir[dir_symbol]), f_out[tangential_offset](mirrored_direction))
 
 
 # end class FreeSlip
@@ -283,7 +291,7 @@ class UBB(LbBoundary):
         Returns:
             list containing LbmWeightInfo and NeighbourOffsetArrays
         """
-        return [LbmWeightInfo(lb_method), NeighbourOffsetArrays(lb_method.stencil)]
+        return [LbmWeightInfo(lb_method, self.data_type), NeighbourOffsetArrays(lb_method.stencil)]
 
     @property
     def velocity_is_callable(self):
@@ -309,22 +317,23 @@ class UBB(LbBoundary):
             cqc = lb_method.conserved_quantity_computation
             shifted_vel_eqs = cqc.equilibrium_input_equations_from_init_values(velocity=velocity)
             shifted_vel_eqs = shifted_vel_eqs.new_without_subexpressions()
-            velocity = [eq.rhs for eq in shifted_vel_eqs.new_filtered(cqc.first_order_moment_symbols).main_assignments]
+            velocity = [eq.rhs for eq in shifted_vel_eqs.new_filtered(cqc.velocity_symbols).main_assignments]
 
         c_s_sq = sp.Rational(1, 3)
-        weight_of_direction = LbmWeightInfo.weight_of_direction
+        weight_info = LbmWeightInfo(lb_method, data_type=self.data_type)
+        weight_of_direction = weight_info.weight_of_direction
         vel_term = 2 / c_s_sq * sum([d_i * v_i for d_i, v_i in zip(neighbor_offset, velocity)]) * weight_of_direction(
             dir_symbol, lb_method)
 
         # Better alternative: in conserved value computation
         # rename what is currently called density to "virtual_density"
         # provide a new quantity density, which is constant in case of incompressible models
-        if not lb_method.conserved_quantity_computation.zero_centered_pdfs:
+        if lb_method.conserved_quantity_computation.compressible:
             cqc = lb_method.conserved_quantity_computation
             density_symbol = sp.Symbol("rho")
             pdf_field_accesses = [f_out(i) for i in range(len(lb_method.stencil))]
             density_equations = cqc.output_equations_from_pdfs(pdf_field_accesses, {'density': density_symbol})
-            density_symbol = lb_method.conserved_quantity_computation.defined_symbols()['density']
+            density_symbol = lb_method.conserved_quantity_computation.density_symbol
             result = density_equations.all_assignments
             result += [Assignment(f_in(inv_dir[dir_symbol]),
                                   f_out(dir_symbol) - vel_term * density_symbol)]
@@ -555,7 +564,7 @@ class FixedDensity(LbBoundary):
             return assignment_collection.copy(new_main_assignments)
 
         cqc = lb_method.conserved_quantity_computation
-        velocity = cqc.defined_symbols()['velocity']
+        velocity = cqc.velocity_symbols
         symmetric_eq = remove_asymmetric_part_of_main_assignments(lb_method.get_equilibrium(),
                                                                   degrees_of_freedom=velocity)
         substitutions = {sym: f_out(i) for i, sym in enumerate(lb_method.pre_collision_pdf_symbols)}
@@ -564,24 +573,26 @@ class FixedDensity(LbBoundary):
         simplification = create_simplification_strategy(lb_method)
         symmetric_eq = simplification(symmetric_eq)
 
-        density_symbol = cqc.defined_symbols()['density']
-
         density = self.density
         equilibrium_input = cqc.equilibrium_input_equations_from_init_values(density=density)
         equilibrium_input = equilibrium_input.new_without_subexpressions()
-        density_eq = equilibrium_input.main_assignments[0]
-        assert density_eq.lhs == density_symbol
-        transformed_density = density_eq.rhs
+        equilibrium_input = equilibrium_input.main_assignments_dict
+
+        subexpressions_dict = symmetric_eq.subexpressions_dict
+        subexpressions_dict[cqc.density_symbol] = equilibrium_input[cqc.density_symbol]
+        subexpressions_dict[cqc.density_deviation_symbol] = equilibrium_input[cqc.density_deviation_symbol]
 
         conditions = [(eq_i.rhs, sp.Equality(dir_symbol, i))
                       for i, eq_i in enumerate(symmetric_eq.main_assignments)] + [(0, True)]
         eq_component = sp.Piecewise(*conditions)
 
-        subexpressions = [Assignment(eq.lhs, transformed_density if eq.lhs == density_symbol else eq.rhs)
-                          for eq in symmetric_eq.subexpressions]
+        main_assignments = [Assignment(f_in(inv_dir[dir_symbol]), 2 * eq_component - f_out(dir_symbol))]
 
-        return subexpressions + [Assignment(f_in(inv_dir[dir_symbol]),
-                                            2 * eq_component - f_out(dir_symbol))]
+        ac = AssignmentCollection(main_assignments, subexpressions=subexpressions_dict)
+        ac = ac.new_without_unused_subexpressions()
+        ac.topological_sort()
+
+        return ac
 
 
 # end class FixedDensity
@@ -595,10 +606,11 @@ class DiffusionDirichlet(LbBoundary):
         name: optional name of the boundary.
     """
 
-    def __init__(self, concentration, name=None):
+    def __init__(self, concentration, name=None, data_type='double'):
         if name is None:
             name = "Diffusion Dirichlet " + str(concentration)
         self.concentration = concentration
+        self._data_type = data_type
 
         super(DiffusionDirichlet, self).__init__(name)
 
@@ -611,10 +623,11 @@ class DiffusionDirichlet(LbBoundary):
         Returns:
             list containing LbmWeightInfo
         """
-        return [LbmWeightInfo(lb_method)]
+        return [LbmWeightInfo(lb_method, self._data_type)]
 
     def __call__(self, f_out, f_in, dir_symbol, inv_dir, lb_method, index_field):
-        w_dir = LbmWeightInfo.weight_of_direction(dir_symbol, lb_method)
+        weight_info = LbmWeightInfo(lb_method, self._data_type)
+        w_dir = weight_info.weight_of_direction(dir_symbol, lb_method)
         return [Assignment(f_in(inv_dir[dir_symbol]),
                            2 * w_dir * self.concentration - f_out(dir_symbol))]
 
