@@ -6,7 +6,8 @@ correction term is added to the collision rule
 import numpy as np
 import sympy as sp
 
-from lbmpy.moments import MOMENT_SYMBOLS
+from lbmpy.moments import MOMENT_SYMBOLS, is_shear_moment, get_order
+from lbmpy.equilibrium import ContinuousHydrodynamicMaxwellian
 from pystencils import Assignment, TypedSymbol
 from pystencils.rng import PhiloxFourFloats, random_symbol
 from pystencils.simp.assignment_collection import SymbolGen
@@ -18,6 +19,8 @@ def add_fluctuations_to_collision_rule(collision_rule, temperature=None, amplitu
     """"""
     if not (temperature and not amplitudes) or (temperature and amplitudes):
         raise ValueError("Fluctuating LBM: Pass either 'temperature' or 'amplitudes'.")
+    if collision_rule.method.conserved_quantity_computation.zero_centered_pdfs:
+        raise ValueError("The fluctuating LBM is not implemented for zero-centered PDF storage.")
 
     method = collision_rule.method
     if not amplitudes:
@@ -60,3 +63,22 @@ def fluctuation_correction(method, rng_generator, amplitudes=SymbolGen("phi")):
 
     # corrections are applied in real space hence we need to convert to real space here
     return method.moment_matrix.inv() * amplitude_matrix * random_matrix
+
+
+class ThermalizedEquilibrium(ContinuousHydrodynamicMaxwellian):
+    """TODO: Remove Again! 
+        
+    This class is currently only used in the tutorial notebook `demo_thermalized_lbm.ipynb`
+    and has been added only temporarily, until the thermalized LBM is updated to our new
+    equilibrium framework."""
+    def __init__(self, random_number_symbols, **kwargs):
+        super().__init__(**kwargs)
+        self.random_number_symbols = random_number_symbols
+
+    def moment(self, exponent_tuple_or_polynomial):
+        value = super().moment(exponent_tuple_or_polynomial)
+        if is_shear_moment(exponent_tuple_or_polynomial, dim=self.dim):
+            value += self.random_number_symbols[0] * 0.001
+        elif get_order(exponent_tuple_or_polynomial) > 2:
+            value += self.random_number_symbols[1] * 0.001
+        return value
